#include "Sleep.h"

#include <chrono>
#include <thread>

#include <ArmarXCore/core/time/DateTime.h>

namespace armarx::skills::provider
{
    Sleep::Sleep() :
        SimpleSpecializedSkill<skills::Example::SleepAcceptedType>(GetSkillDescription())
    {
    }

    SkillDescription
    Sleep::GetSkillDescription()
    {
        armarx::skills::Example::SleepAcceptedType root_profile_params;
        root_profile_params.Timeout_ms = 5000;

        return SkillDescription{
            .skillId = skills::SkillID{.skillName = "Sleep"},
            .description =
                "This skill exits after the provided time (or five minutes, whichever is lower).",
            .rootProfileDefaults = root_profile_params.toAron(),
            .timeout = armarx::core::time::Duration::Minutes(5),
            .parametersType =
                armarx::skills::Example::SleepAcceptedType::SleepAcceptedType::ToAronType()};
    }

    void
    Sleep::onStopRequested()
    {
        aborted = true;
    }

    Skill::MainResult
    Sleep::main(const SpecializedMainInput& in)
    {
        timeout = armarx::Duration::MilliSeconds(in.parameters.Timeout_ms);
        auto start = armarx::DateTime::Now();
        for (;;) // forever
        {
            if (aborted)
            {
                ARMARX_INFO << "Skill has been woken up by abort.";
                return Skill::MakeAbortedResult();
            }

            // check if the time is over
            auto now = armarx::DateTime::Now();
            auto diff = now - start;
            if (diff >= timeout)
            {
                ARMARX_INFO << "Skill has woken up by timeout.";
                return Skill::MakeSucceededResult();
            }
            else
            {
                std::this_thread::sleep_for(std::chrono::milliseconds(100));
            }
        }

        return {TerminatedSkillStatus::Succeeded, in.parameters.toAron()};
    }


} // namespace armarx::skills::provider
