#pragma once

#include <filesystem>
#include <optional>

#include <ArmarXCore/core/services/tasks/TaskUtil.h>

#include "RobotAPI/libraries/ArmarXObjects/ObjectID.h"
#include "RobotAPI/libraries/ArmarXObjects/ObjectInfo.h"
#include "RobotAPI/libraries/armem/server/forward_declarations.h"
#include "RobotAPI/libraries/armem/server/segment/SpecializedProviderSegment.h"
#include <RobotAPI/libraries/armem_grasping/aron/KnownGraspCandidate.aron.generated.h>

namespace armarx::armem::grasping::segment
{
    class KnownGraspProviderSegment : public armem::server::segment::SpecializedProviderSegment
    {
        using Base = armem::server::segment::SpecializedProviderSegment;

    public:
        KnownGraspProviderSegment(armem::server::MemoryToIceAdapter& iceMemory);

        void init() override;

    private:
        void loadMemory();
        std::optional<arondto::KnownGraspInfo> knownGraspInfoFromObjectInfo(const ObjectInfo&);

        struct GraspFileInfo
        {
            PackageFileLocation fileLocInfo;
            ObjectID objectId;

            static GraspFileInfo FromObjectInfo(const ObjectInfo& objectInfo)
            {
                const std::string objectClassName = objectInfo.className();
                PackageFileLocation fileLocInfo = objectInfo.file(".xml", "_Grasps");

                return GraspFileInfo{.fileLocInfo = fileLocInfo, .objectId = objectInfo.id()};
            }
        };

        std::optional<arondto::KnownGraspInfo>
        knownGraspInfoFromFile(const GraspFileInfo& graspFileInfo);

        void installFileWatcher();

        SimpleRunningTask<>::pointer_type fileWatcherTask;

    public:
        static const constexpr char* CORE_SEGMENT_NAME = "KnownGraspCandidate";
        static const constexpr char* PROVIDER_SEGMENT_NAME = "PriorKnowledgeData";

    private:
        static const constexpr char* PREPOSE_SUFFIX = "_Prepose";
        static const constexpr char* GRASP_OPTIONAL_SUFFIX = "_Grasp";
    };
} // namespace armarx::armem::grasping::segment
