#pragma once

#include <experimental/memory>
#include <optional>
#include <string>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioNode.aron.generated.h>

namespace armarx
{
    namespace skills
    {
        enum class FluxioNodeType
        {
            UNKNOWN = 0,
            PARAMETER = 1,
            CONTROL = 2,
            SUBSKILL = 3,
        };

        struct FluxioNode
        {
            virtual ~FluxioNode() = default;
            FluxioNodeType nodeType;
            std::string nodeId;
            std::string name;
            float xPos = 0;
            float yPos = 0;

            virtual std::optional<manager::dto::FluxioNode> toManagerIce() const;
            manager::dto::FluxioIdentificator toFluxioIdentificatorIce() const;
            virtual std::optional<manager::arondto::FluxioNode> toAron() const;
            manager::arondto::FluxioIdentificator toFluxioIdentificatorAron() const;

            static std::experimental::observer_ptr<const FluxioNode> FromFluxioIdentificatorIce(
                const manager::dto::FluxioIdentificator& i,
                const std::map<const std::string, const std::unique_ptr<FluxioNode>>& nodesMap);
            static std::experimental::observer_ptr<const FluxioNode> FromFluxioIdentificatorAron(
                const manager::arondto::FluxioIdentificator& i,
                const std::map<const std::string, const std::unique_ptr<FluxioNode>>& nodesMap);
            static FluxioNode FromIce(const manager::dto::FluxioNode& i);
        };

        FluxioNodeType FluxioNodeTypeFromString(const std::string& type);
        std::optional<std::string> FluxioNodeTypeToString(const FluxioNodeType& type);
    } // namespace skills
} // namespace armarx
