#pragma once

#include <experimental/memory>
#include <list>
#include <string>
#include <vector>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include "RobotAPI/libraries/aron/core/type/variant/Variant.h"
#include "RobotAPI/libraries/aron/core/type/variant/container/Object.h"
#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioParameter.aron.generated.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioTypeIdentificator.aron.generated.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioValue.aron.generated.h>

#include "FluxioTypeIdentificator.h"
#include "FluxioValue.h"

namespace armarx
{
    namespace skills
    {
        struct FluxioParameter
        {
            std::string id;
            std::string name;
            std::string description = "";
            FluxioTypeIdentificator typeIdentificator;
            aron::type::VariantPtr type;
            bool required = true;
            bool isInput;
            std::list<FluxioValue> values;

            manager::dto::FluxioParameter toManagerIce() const;
            manager::dto::FluxioIdentificator toFluxioIdentificatorIce() const;
            manager::arondto::FluxioParameter toAron() const;
            manager::arondto::FluxioIdentificator toFluxioIdentificatorAron() const;
            void updateFromIce(const manager::dto::FluxioParameter& i,
                               std::map<std::string, FluxioProfile>& profilesMap,
                               std::map<std::string, aron::type::ObjectPtr>& typesMap);
            void updateFromAron(const manager::arondto::FluxioParameter& i,
                                std::map<std::string, FluxioProfile>& profilesMap,
                                std::map<std::string, aron::type::ObjectPtr>& typesMap);
            void updateValuesFromIce(const manager::dto::FluxioValueList& i,
                                     std::map<std::string, FluxioProfile>& profilesMap);
            void updateValuesFromAron(const std::vector<manager::arondto::FluxioValue>& i,
                                     std::map<std::string, FluxioProfile>& profilesMap);

            static std::experimental::observer_ptr<const FluxioParameter>
            FromFluxioIdentificatorIce(const manager::dto::FluxioIdentificator& i,
                                       const std::map<std::string, FluxioParameter>& parametersMap);
            static std::experimental::observer_ptr<const FluxioParameter>
            FromFluxioIdentificatorAron(
                const manager::arondto::FluxioIdentificator& i,
                const std::map<std::string, FluxioParameter>& parametersMap);
            static FluxioParameter FromIce(const manager::dto::FluxioParameter& i,
                                           std::map<std::string, FluxioProfile>& profilesMap,
                                           std::map<std::string, aron::type::ObjectPtr>& typesMap);
            static FluxioParameter FromAron(const manager::arondto::FluxioParameter& i,
                                            std::map<std::string, FluxioProfile>& profilesMap,
                                            std::map<std::string, aron::type::ObjectPtr>& typesMap);
        };
    } // namespace skills
} // namespace armarx
