#include "FluxioParameterNode.h"

#include <optional>

#include <ArmarXCore/core/logging/Logging.h>

#include "RobotAPI/libraries/skills/core/FluxioNode.h"
#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioNode.aron.generated.h>

#include "FluxioParameter.h"

namespace armarx
{
    namespace skills
    {
        std::optional<manager::dto::FluxioNode>
        FluxioParameterNode::toManagerIce() const
        {
            if (parameterPtr == nullptr)
            {
                ARMARX_WARNING << "Parameter for Parameternode with id " << nodeId << " not set";
                return std::nullopt;
            }

            const auto& nt = FluxioNodeTypeToString(nodeType);
            if (!nt.has_value())
            {
                return std::nullopt;
            }

            manager::dto::FluxioNode ret;
            ret.nodeId = nodeId;
            ret.nodeType = nt.value();
            ret.name = name;
            ret.xPos = xPos;
            ret.yPos = yPos;
            ret.parameterId = parameterPtr->toFluxioIdentificatorIce();

            manager::dto::FluxioIdentificator subSId;
            subSId.id = "";
            subSId.hint = "";
            ret.skillId = subSId;

            return ret;
        }

        std::optional<manager::arondto::FluxioNode>
        FluxioParameterNode::toAron() const
        {
            if (parameterPtr == nullptr)
            {
                ARMARX_WARNING << "Parameter for Parameternode with id " << nodeId << " not set";
                return std::nullopt;
            }

            const auto& nt = FluxioNodeTypeToString(nodeType);
            if (!nt.has_value())
            {
                return std::nullopt;
            }

            manager::arondto::FluxioNode ret;
            ret.nodeId = nodeId;
            ret.nodeType = nt.value();
            ret.name = name;
            ret.xPos = xPos;
            ret.yPos = yPos;
            ret.parameterId = parameterPtr->toFluxioIdentificatorAron();

            manager::arondto::FluxioIdentificator subSId;
            subSId.id = "";
            subSId.hint = "";
            ret.skillId = subSId;

            return ret;
        }

        std::optional<FluxioParameterNode>
        FluxioParameterNode::FromIce(const manager::dto::FluxioNode& i,
                                     std::map<std::string, FluxioParameter>& parametersMap)
        {
            if (i.parameterId.id.empty())
            {
                ARMARX_WARNING << "Parameters id is dummy id in ParameterNode with id " << i.nodeId;
                return std::nullopt;
            }
            const auto& parameterPtr =
                FluxioParameter::FromFluxioIdentificatorIce(i.parameterId, parametersMap);

            if (parameterPtr == nullptr)
            {
                return std::nullopt;
            }

            FluxioParameterNode ret;

            ret.nodeId = i.nodeId;
            ret.nodeType = FluxioNodeTypeFromString(i.nodeType);
            ret.name = i.name;
            ret.xPos = i.xPos;
            ret.yPos = i.yPos;
            ret.parameterPtr = parameterPtr;

            return ret;
        }

        std::optional<FluxioParameterNode>
        FluxioParameterNode::FromAron(const manager::arondto::FluxioNode& i,
                                     std::map<std::string, FluxioParameter>& parametersMap)
        {
            if (i.parameterId.id.empty())
            {
                ARMARX_WARNING << "Parameters id is dummy id in ParameterNode with id " << i.nodeId;
                return std::nullopt;
            }
            const auto& parameterPtr =
                FluxioParameter::FromFluxioIdentificatorAron(i.parameterId, parametersMap);

            if (parameterPtr == nullptr)
            {
                return std::nullopt;
            }

            FluxioParameterNode ret;

            ret.nodeId = i.nodeId;
            ret.nodeType = FluxioNodeTypeFromString(i.nodeType);
            ret.name = i.name;
            ret.xPos = i.xPos;
            ret.yPos = i.yPos;
            ret.parameterPtr = parameterPtr;

            return ret;
        }
    } // namespace skills
} // namespace armarx
