#include "FluxioProfile.h"

#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioIdentificator.aron.generated.h>

namespace armarx
{
    namespace skills
    {
        manager::dto::FluxioProfile
        FluxioProfile::toManagerIce() const
        {
            manager::dto::FluxioProfile ret;

            ret.id = id;
            ret.name = name;
            ret.description = description;
            if (parentPtr == nullptr)
            {
                ret.parentIdHasValue = false;
                ret.parentId.id = "";
                ret.parentId.hint = "";
            }
            else
            {
                ret.parentIdHasValue = true;
                ret.parentId = parentPtr->toFluxioIdentificatorIce();
            }

            return ret;
        }

        manager::arondto::FluxioProfile
        FluxioProfile::toManagerAron() const
        {
            manager::arondto::FluxioProfile ret;

            ret.id = id;
            ret.name = name;
            ret.description = description;
            if (parentPtr == nullptr)
            {
                ret.parentIdHasValue = false;
                ret.parentId.id = "";
                ret.parentId.hint = "";
            }
            else
            {
                ret.parentIdHasValue = true;
                ret.parentId = parentPtr->toFluxioIdentificatorAron();
            }

            return ret;
        }

        manager::dto::FluxioIdentificator
        FluxioProfile::toFluxioIdentificatorIce() const
        {
            manager::dto::FluxioIdentificator ret;

            ret.id = id;
            ret.hint = name;

            return ret;
        }

        manager::arondto::FluxioIdentificator
        FluxioProfile::toFluxioIdentificatorAron() const
        {
            manager::arondto::FluxioIdentificator ret;

            ret.id = id;
            ret.hint = name;

            return ret;
        }

        std::experimental::observer_ptr<const FluxioProfile>
        FluxioProfile::FromFluxioIdentificatorIce(const manager::dto::FluxioIdentificator& i,
                                                  std::map<std::string, FluxioProfile>& profilesMap)
        {
            const auto& profilesEntry = profilesMap.find(i.id);

            if (profilesEntry == profilesMap.end())
            {
                ARMARX_WARNING << "Profile with id " << i.id << " not found";
                return nullptr;
            }

            return std::experimental::make_observer(&(profilesMap[profilesEntry->second.id]));
        }

        std::experimental::observer_ptr<const FluxioProfile>
        FluxioProfile::FromFluxioIdentificatorAron(const manager::arondto::FluxioIdentificator& i,
                                                  std::map<std::string, FluxioProfile>& profilesMap)
        {
            const auto& profilesEntry = profilesMap.find(i.id);

            if (profilesEntry == profilesMap.end())
            {
                ARMARX_WARNING << "Profile with id " << i.id << " not found";
                return nullptr;
            }

            return std::experimental::make_observer(&(profilesMap[profilesEntry->second.id]));
        }

        FluxioProfile
        FluxioProfile::FromIce(const manager::dto::FluxioProfile& i,
                               std::map<std::string, FluxioProfile>& profilesMap)
        {
            // TODO: find a way to manage a list of import, so that the sorting of the list will not be problematic
            FluxioProfile ret;

            ret.id = i.id;
            ret.name = i.name;
            ret.description = i.description;
            if (!i.parentIdHasValue)
            {
                ret.parentPtr = nullptr;
            }
            else
            {
                ret.parentPtr = FromFluxioIdentificatorIce(i.parentId, profilesMap);
            }

            return ret;
        }
    } // namespace skills
} // namespace armarx
