#include "FluxioProvider.h"

#include <experimental/memory>
#include <memory>

#include <ArmarXCore/core/logging/Logging.h>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include "RobotAPI/libraries/aron/core/data/variant/forward_declarations.h"
#include "RobotAPI/libraries/aron/core/data/variant/primitive/String.h"

namespace armarx
{
    namespace skills
    {
        manager::dto::FluxioProvider
        FluxioProvider::toManagerIce() const
        {
            manager::dto::FluxioProvider ret;

            ret.id = id;
            ret.name = name;

            return ret;
        }

        manager::dto::FluxioIdentificator
        FluxioProvider::toFluxioIdentificatorIce() const
        {
            manager::dto::FluxioIdentificator ret;

            ret.id = id;
            ret.hint = name;

            return ret;
        }

        manager::arondto::FluxioIdentificator
        FluxioProvider::toFluxioIdentificatorAron() const
        {
            manager::arondto::FluxioIdentificator ret;

            ret.id = id;
            ret.hint = name;

            return ret;
        }

        std::experimental::observer_ptr<const FluxioProvider>
        FluxioProvider::FromFluxioIdentificatorIce(
            const manager::dto::FluxioIdentificator& i,
            std::map<std::string, FluxioProvider>& providersMap)
        {
            const auto& providerIt = providersMap.find(i.id);

            if (providerIt == providersMap.end())
            {
                ARMARX_WARNING << "Provider with id " << i.id << " not found";
                return nullptr;
            }

            return std::experimental::make_observer(&(providersMap[providerIt->first]));
        }

        
        std::experimental::observer_ptr<const FluxioProvider>
        FluxioProvider::FromFluxioIdentificatorAron(
            const manager::arondto::FluxioIdentificator& i,
            std::map<std::string, FluxioProvider>& providersMap)
        {
            const auto& providerIt = providersMap.find(i.id);

            if (providerIt == providersMap.end())
            {
                ARMARX_WARNING << "Provider with id " << i.id << " not found";
                return nullptr;
            }

            return std::experimental::make_observer(&(providersMap[providerIt->first]));
        }

        FluxioProvider
        FluxioProvider::FromIce(const manager::dto::FluxioProvider& i)
        {
            FluxioProvider ret;

            ret.id = i.id;
            ret.name = i.name;

            return ret;
        }

        aron::data::DictPtr
        FluxioProvider::toAron() const
        {
            aron::data::DictPtr dict = aron::make_dict();
            dict->addElement("id", aron::make_string(this->id));
            dict->addElement("hint", aron::make_string(this->name));
            return dict;
        }
    } // namespace skills
} // namespace armarx
