#pragma once

#include <memory>
#include <stdexcept>

namespace armarx
{
    namespace skills
    {

        template <typename T, typename E = std::exception>
        class Result
        {
        private:
            const bool success;
            const std::shared_ptr<const T> result;
            const std::shared_ptr<const E> error;

        public:
            Result(const T& res) :
                success(true), result(std::make_shared<const T>(res)), error(nullptr)
            {
            }

            Result(const E& err) :
                success(false), result(nullptr), error(std::make_shared<const E>(err))
            {
            }

            bool
            isSuccess() const
            {
                return success;
            }

            T
            getResult() const
            {
                if (!success)
                {
                    throw std::logic_error("Result is not successful");
                }
                return *result;
            }

            E
            getError() const
            {
                if (success)
                {
                    throw std::logic_error("Result does not contain an error");
                }
                return *error;
            }
        };
    } // namespace skills
} // namespace armarx
