#pragma once

#include <list>
#include <memory>
#include <optional>
#include <string>

#include <ArmarXCore/core/time/Duration.h>

#include <RobotAPI/interface/aron/Aron.h>
#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioIdentificator.aron.generated.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioNode.aron.generated.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioSkill.aron.generated.h>

#include "FluxioEdge.h"
#include "FluxioNode.h"
#include "FluxioParameter.h"
#include "FluxioProvider.h"

namespace armarx
{
    namespace skills
    {
        struct FluxioSkill
        {
            std::string id;
            std::string name;
            std::string description;
            /**
             * @brief How long (in ms) to wait for the skill to finish execution before timing out. A negative value indicates no timeout (default).
             */
            armarx::Duration timeout = armarx::Duration::MilliSeconds(-1);
            std::string lastChanged;
            bool executable =
                false; // TODO: change from explicit to implicit storing of executable state
            bool native = true;
            std::experimental::observer_ptr<const FluxioProvider> skillProviderPtr;
            std::map<std::string, FluxioParameter> parameters;
            std::map<const std::string, const std::unique_ptr<FluxioNode>> nodes;
            std::list<FluxioEdge> edges;

            FluxioSkill(const FluxioSkill&) = delete;
            FluxioSkill& operator=(const FluxioSkill&) = delete;
            FluxioSkill(FluxioSkill&&) = default;
            FluxioSkill() = default;

            std::optional<manager::dto::FluxioSkill> toManagerIce() const;
            std::optional<aron::data::dto::DictPtr> toAronDTO() const;
            std::optional<manager::arondto::FluxioSkill> toAronXml() const;
            manager::dto::FluxioIdentificator toFluxioIdentificatorIce() const;
            manager::arondto::FluxioIdentificator toFluxioIdentificatorAron() const;
            bool updateFromIce(const manager::dto::FluxioSkill& i,
                               std::map<std::string, FluxioProvider>& providersMap,
                               std::map<std::string, FluxioProfile>& profilesMap,
                               std::map<std::string, FluxioSkill>& skillsMap,
                               std::map<std::string, aron::type::ObjectPtr>& typesMap);
            bool updateFromAron(const aron::data::dto::DictPtr& i,
                                std::map<std::string, FluxioProvider>& providersMap,
                                std::map<std::string, FluxioProfile>& profilesMap,
                                std::map<std::string, FluxioSkill>& skillsMap,
                                std::map<std::string, aron::type::ObjectPtr>& typesMap);
            void deleteParameter(const std::string& parameterId);
            void removeParameterNodesAndEdges(const std::string& parameterId,
                                              bool keepParameterNodes = false);
            void removeSubSkillNodesAndEdges(const std::string& skillId);
            void removeEdgesConnectedToParameter(const std::string& parameterId);

            static std::experimental::observer_ptr<const FluxioSkill>
            FromFluxioIdentificatorIce(const manager::dto::FluxioIdentificator& i,
                                       std::map<std::string, FluxioSkill>& skillsMap);
            static std::experimental::observer_ptr<const FluxioSkill>
            FromFluxioIdentificatorAron(const manager::arondto::FluxioIdentificator& i,
                                        std::map<std::string, FluxioSkill>& skillsMap);
            static std::unique_ptr<FluxioSkill>
            FromIce(const manager::dto::FluxioSkill& i,
                    std::map<std::string, FluxioProvider>& providersMap,
                    std::map<std::string, FluxioProfile>& profilesMap,
                    std::map<std::string, FluxioSkill>& skillsMap,
                    std::map<std::string, aron::type::ObjectPtr>& typesMap);
            static std::unique_ptr<FluxioSkill>
            FromAron(const aron::data::dto::DictPtr& d,
                     std::map<std::string, FluxioProvider>& providersMap,
                     std::map<std::string, FluxioProfile>& profilesMap,
                     std::map<std::string, FluxioSkill>& skillsMap,
                     std::map<std::string, aron::type::ObjectPtr>& typesMap);
            static std::unique_ptr<skills::FluxioNode>
            CreateNode(const manager::dto::FluxioNode& i,
                       std::map<std::string, FluxioParameter>& parametersMap,
                       std::map<std::string, FluxioSkill>& skillsMap,
                       std::map<std::string, FluxioProfile>& profilesMap,
                       std::map<std::string, aron::type::ObjectPtr>& typesMap);
            static std::unique_ptr<skills::FluxioNode>
            CreateNode(const manager::arondto::FluxioNode& i,
                       std::map<std::string, FluxioParameter>& parametersMap,
                       std::map<std::string, FluxioSkill>& skillsMap,
                       std::map<std::string, FluxioProfile>& profilesMap,
                       std::map<std::string, aron::type::ObjectPtr>& typesMap);
        };
    } // namespace skills
} // namespace armarx
