#include "FluxioSubSkillNode.h"

#include <ArmarXCore/core/logging/Logging.h>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include "RobotAPI/libraries/aron/core/data/variant/primitive/Float.h"
#include "RobotAPI/libraries/aron/core/data/variant/primitive/String.h"
#include "RobotAPI/libraries/skills/core/FluxioNode.h"
#include <RobotAPI/interface/skills/SkillManagerInterface.h>
#include <RobotAPI/libraries/skills/core/aron/FluxioNode.aron.generated.h>

#include "FluxioSkill.h"

namespace armarx
{
    namespace skills
    {
        std::optional<manager::dto::FluxioNode>
        FluxioSubSkillNode::toManagerIce() const
        {
            if (skillPtr == nullptr)
            {
                ARMARX_WARNING << "Skill for SubSkillNode with id " << nodeId << " not set";
                return std::nullopt;
            }

            const auto& nt = FluxioNodeTypeToString(nodeType);
            if (!nt.has_value())
            {
                return std::nullopt;
            }

            manager::dto::FluxioNode ret;
            ret.nodeId = nodeId;
            ret.nodeType = nt.value();
            ret.name = name;
            ret.xPos = xPos;
            ret.yPos = yPos;

            manager::dto::FluxioIdentificator paramId;
            paramId.id = "";
            paramId.hint = "";
            ret.parameterId = paramId;

            ret.skillId = skillPtr->toFluxioIdentificatorIce();

            return ret;
        }

        std::optional<manager::arondto::FluxioNode>
        FluxioSubSkillNode::toAron() const
        {
            if (skillPtr == nullptr)
            {
                ARMARX_WARNING << "Skill for SubSkillNode with id " << nodeId << " not set";
                return std::nullopt;
            }

            const auto& nt = FluxioNodeTypeToString(nodeType);
            if (!nt.has_value())
            {
                return std::nullopt;
            }

            manager::arondto::FluxioNode ret;
            ret.nodeId = nodeId;
            ret.nodeType = nt.value();
            ret.name = name;
            ret.xPos = xPos;
            ret.yPos = yPos;

            manager::arondto::FluxioIdentificator paramId;
            paramId.id = "";
            paramId.hint = "";
            ret.parameterId = paramId;

            ret.skillId = skillPtr->toFluxioIdentificatorAron();

            return ret;
        }

        std::optional<FluxioSubSkillNode>
        FluxioSubSkillNode::FromIce(const manager::dto::FluxioNode& i,
                                    std::map<std::string, FluxioSkill>& skillsMap)
        {
            if (i.skillId.id.empty())
            {
                ARMARX_WARNING << "Skills id is dummy id in SubSkillNode with id " << i.nodeId;
                return std::nullopt;
            }
            const auto& skillPtr = FluxioSkill::FromFluxioIdentificatorIce(i.skillId, skillsMap);

            if (skillPtr == nullptr)
            {
                ARMARX_WARNING << "Skill for SubSkillNode with id " << i.nodeId << " not found";
                return std::nullopt;
            }

            FluxioSubSkillNode ret;
            ret.nodeId = i.nodeId;
            ret.nodeType = FluxioNodeTypeFromString(i.nodeType);
            ret.name = i.name;
            ret.xPos = i.xPos;
            ret.yPos = i.yPos;
            ret.skillPtr = skillPtr;

            return ret;
        }

        std::optional<FluxioSubSkillNode>
        FluxioSubSkillNode::FromAron(const manager::arondto::FluxioNode& i,
                                    std::map<std::string, FluxioSkill>& skillsMap)
        {
            if (i.skillId.id.empty())
            {
                ARMARX_WARNING << "Skills id is dummy id in SubSkillNode with id " << i.nodeId;
                return std::nullopt;
            }
            const auto& skillPtr = FluxioSkill::FromFluxioIdentificatorAron(i.skillId, skillsMap);

            if (skillPtr == nullptr)
            {
                ARMARX_WARNING << "Skill for SubSkillNode with id " << i.nodeId << " not found";
                return std::nullopt;
            }

            FluxioSubSkillNode ret;
            ret.nodeId = i.nodeId;
            ret.nodeType = FluxioNodeTypeFromString(i.nodeType);
            ret.name = i.name;
            ret.xPos = i.xPos;
            ret.yPos = i.yPos;
            ret.skillPtr = skillPtr;

            return ret;
        }
    } // namespace skills
} // namespace armarx
