#include "FluxioValue.h"

#include <optional>

#include <ArmarXCore/core/logging/Logging.h>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include <RobotAPI/libraries/skills/core/aron/FluxioValue.aron.generated.h>

#include "FluxioProfile.h"

namespace armarx
{
    namespace skills
    {
        std::optional<manager::dto::FluxioValue>
        FluxioValue::toManagerIce() const
        {
            manager::dto::FluxioValue ret;

            if (profilePtr == nullptr)
            {
                ARMARX_WARNING << "Profile not set.";
                return std::nullopt;
            }

            ret.profileId = profilePtr->toFluxioIdentificatorIce();
            if (content == nullptr)
            {
                // ARMARX_WARNING << "Content not set.";
                return std::nullopt;
            }
            ret.content = content->toAronDTO();

            return ret;
        }

        std::optional<FluxioValue>
        FluxioValue::FromIce(const manager::dto::FluxioValue& i,
                             std::map<std::string, FluxioProfile>& profilesMap)
        {
            auto profilePtr = FluxioProfile::FromFluxioIdentificatorIce(i.profileId, profilesMap);

            if (profilePtr == nullptr)
            {
                return std::nullopt;
            }

            FluxioValue ret;

            ret.profilePtr = profilePtr;
            ret.content = armarx::aron::data::Variant::FromAronDTO(i.content);

            return ret;
        }

        std::optional<FluxioValue>
        FluxioValue::FromAron(const manager::arondto::FluxioValue& i,
                             std::map<std::string, FluxioProfile>& profilesMap)
        {
            auto profilePtr = FluxioProfile::FromFluxioIdentificatorAron(i.profileId, profilesMap);

            if (profilePtr == nullptr)
            {
                return std::nullopt;
            }

            FluxioValue ret;

            ret.profilePtr = profilePtr;
            if (!i.content->hasElement("content")) {
                ARMARX_WARNING << "Error while converting FluxioValue from Aron for profile: '" << i.profileId.id << "': content not found";
                return std::nullopt;
            }
            ret.content = i.content->getElement("content");

            return ret;
        }

        std::optional<manager::arondto::FluxioValue>
        FluxioValue::toAron() const
        {
            manager::arondto::FluxioValue ret;

            
            if (profilePtr == nullptr)
            {
                ARMARX_WARNING << "Profile not set.";
                return std::nullopt;
            }

            ret.profileId = profilePtr->toFluxioIdentificatorAron();
            if (content == nullptr)
            {
                // ARMARX_WARNING << "Content not set.";
                return std::nullopt;
            }

            const aron::data::DictPtr& dict = aron::make_dict();
            dict->addElement("content", content);
            ret.content = dict;

            return ret;
        }

    } // namespace skills
} // namespace armarx
