/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <string>

#include <ArmarXCore/core/exceptions/LocalException.h>

namespace armarx::skills::error
{
    /**
     * @brief A base class for skill exceptions. All skill exceptions inherit from this class
     */
    class SkillException : public armarx::LocalException
    {
    public:
        SkillException() = delete;

        SkillException(const std::string& prettymethod, const std::string& reason);
    };

    /**
     * @brief Indicates that a skill was not found, e.g., by the skill manager.
     */
    class SkillNotFoundException : public SkillException
    {
    public:
        SkillNotFoundException() = delete;

        SkillNotFoundException(const std::string& prettymethod, const std::string& reason);
    };

    class SkillAbortedException : public SkillException
    {
    public:
        SkillAbortedException() = delete;

        SkillAbortedException(const std::string& prettymethod, const std::string& reason);
    };

    class SkillFailedException : public SkillException
    {
    public:
        SkillFailedException() = delete;

        SkillFailedException(const std::string& prettymethod, const std::string& reason);
    };

    /**
     * @brief The NotImplementedYetException class
     */
    class NotImplementedYetException : public SkillException
    {
    public:
        NotImplementedYetException() = delete;

        NotImplementedYetException(const std::string& prettymethod);
    };
} // namespace armarx::skills::error
