#include "FluxioErrorMessages.h"

#include <string>

namespace armarx::skills::error
{
    std::string
    formatString(const std::string& format, const std::vector<std::string>& args)
    {
        std::string formatted = format;
        size_t pos = 0;
        size_t argIndex = 0;

        while ((pos = formatted.find("%s", pos)) != std::string::npos && argIndex < args.size())
        {
            formatted.replace(pos, 2, args[argIndex]);
            pos += args[argIndex].length();
            ++argIndex;
        }

        return formatted;
    }

    std::string
    createErrorMessage(ErrorCode code, const std::vector<std::string>& args)
    {
        switch (code)
        {
            case ErrorCode::ExecutionNotFound:
                return formatString("Execution with ID '%s' not found.", args);
            case ErrorCode::ExecutorNotFound:
                return formatString("Executor with ID '%s' not found.", args);
            case ErrorCode::RunnerNotFound:
                return formatString(
                    "Error while getting execution runner for fluxio skill with id '%s'", args);
            case ErrorCode::ConverterError:
                return formatString("Skill with ID '%s' could not be converted.", args);
            case ErrorCode::NotFluxioCompExecutor:
                return formatString("Executor with ID '%s' is not a FluxioCompositeExecutor.",
                                    args);
            case ErrorCode::SkillNotFound:
                return formatString("Skill with ID '%s' not found.", args);
            case ErrorCode::ParameterNotFound:
                return formatString("Parameter with ID '%s' not found.", args);
            case ErrorCode::ProfileNotFound:
                return formatString("Profile with ID '%s' not found.", args);
            case ErrorCode::ProviderNotFound:
                return formatString("Provider with ID '%s' not found.", args);
            case ErrorCode::SkillDescNotFound:
                return formatString(
                    "Skill description for Skill with ID '%s' not found. Abort executing.", args);
            case ErrorCode::GenericMutexError:
                return formatString("Mutex for Skill with ID '%s' could not be aquired", args);
            case ErrorCode::UserHasNoMutexError:
                return formatString("Mutex for Skill with ID '%s' could not be removed because the "
                                    "user does not hold the mutex.",
                                    args);
            case ErrorCode::AddSkillError:
                return formatString(
                    "Skill with ID: '%s' could not be added to provider with ID: '%s'.", args);
            case ErrorCode::TestError:
                return formatString("This is a test Error with Argument: '%s'", args);
            case ErrorCode::ProviderAlreadyExists:
                return formatString("Provider with ID '%s' already exists.", args);
            case ErrorCode::ProfileHasNoParent:
                return formatString("Profile with name: '%s' could not be added because it has no "
                                    "Parent Profile assigned.",
                                    args);
            case ErrorCode::ProfileAlreadyExists:
                return formatString("Profile with name: '%s' already exists.", args);
            case ErrorCode::MissingRequired:
                return formatString("Required '%s' is missing.", args);
            default:
                return "Unknown error.";
        }
    }
} // namespace armarx::skills::error
