#pragma once

#include <optional>
#include <string>

#include <ArmarXCore/core/exceptions/Exception.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>

#include <RobotAPI/interface/skills/SkillManagerInterface.h>

namespace armarx::skills::error
{
    enum class FluxioExceptionType
    {
        BAD_REQUEST,
        UNAUTHORIZED,
        FORBIDDEN,
        NOT_FOUND
    };

    /**
     * @brief A base class for skill exceptions. All skill exceptions inherit from this class
     */
    class FluxioException : public armarx::LocalException
    {
    private:
        const std::string message;
        FluxioExceptionType type;
        std::vector<std::string> contextList;

    public:
        FluxioException() = delete;

        FluxioException(const std::string& message,
                        const FluxioExceptionType& type,
                        const std::vector<std::string>& contextList = {}) :
            message(message), type(type), contextList(contextList)
        {
        }

        void addToContext(const std::optional<std::string>& contextItem = std::nullopt,
                          const std::optional<std::string>& className = std::nullopt,
                          const std::optional<std::string>& methodName = std::nullopt,
                          const std::optional<int>& line = std::nullopt);

        static FluxioException create(const std::string& message,
                                      const FluxioExceptionType& type,
                                      const std::string& className,
                                      const char* function,
                                      int line);

        skills::manager::dto::FluxioException toManagerIce() const;
        static std::string FluxioExceptionTypeToString(const FluxioExceptionType& type);
    };

} // namespace armarx::skills::error
