#pragma once

#include <experimental/memory>
#include <list>
#include <map>
#include <optional>
#include <shared_mutex>
#include <string>
#include <vector>

#include "RobotAPI/libraries/aron/core/data/variant/Variant.h"
#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include "RobotAPI/libraries/skills/core/FluxioParameter.h"
#include "RobotAPI/libraries/skills/core/FluxioProfile.h"
#include "RobotAPI/libraries/skills/core/SkillStatusUpdate.h"

#include "../FluxioSkillStatusUpdate.h"

namespace armarx::skills
{
    class FluxioExecutor
    {
    public:
        virtual ~FluxioExecutor(){};
        // Copy constructor and assignment operator would be implicitly deleted because of the mutexes
        // this is just to make it explicit for better error messages O_o
        FluxioExecutor(const FluxioExecutor&) = delete;
        FluxioExecutor& operator=(const FluxioExecutor&) = delete;

        FluxioExecutor(const std::string& id, bool native) : id(id), native(native){};
        virtual void run(const std::string executorName,
                         armarx::aron::data::DictPtr parameters,
                         const std::experimental::observer_ptr<const FluxioProfile> profilePtr) {};
        virtual void abort() {};
        virtual armarx::aron::data::DictPtr getResultsCopy();
        virtual armarx::aron::data::VariantPtr getPossibleInputCopy(const std::string nodeId,
                                                                    const std::string parameterId);

        virtual std::optional<std::vector<skills::FluxioSkillStatusUpdate>> getStatusUpdate();

        std::optional<skills::FluxioSkillStatusUpdate> getStatus() const;

        armarx::aron::data::VariantPtr
        findParameterValue(const std::experimental::observer_ptr<const FluxioProfile> profilePtr,
                           const FluxioParameter& parameter) const;

        const std::string id;
        const bool native;

    protected:
        std::optional<std::string> executorName = std::nullopt;

        std::list<skills::FluxioSkillStatusUpdate> statusUpdates;
        std::shared_mutex statusUpdatesMutex;

        armarx::aron::data::DictPtr result = nullptr;
        std::shared_mutex resultMutex;

        std::map<std::string, armarx::aron::data::DictPtr> possibleInputs = {};
        std::shared_mutex possibleInputsMutex;

        std::optional<skills::FluxioSkillStatusUpdate> status = std::nullopt;
        std::shared_mutex statusMutex;
        virtual void setStatus(skills::SkillStatus status, const std::string& nodeId = "noId");
    };
} // namespace armarx::skills
