#pragma once

#include <optional>
#include <string>

#include "RobotAPI/libraries/aron/core/data/variant/container/Dict.h"
#include "RobotAPI/libraries/skills/core/FluxioSkill.h"
#include "RobotAPI/libraries/skills/core/SkillExecutionRequest.h"
#include "RobotAPI/libraries/skills/core/SkillID.h"
#include "RobotAPI/libraries/skills/core/SkillStatusUpdate.h"

#include "../FluxioSkillStatusUpdate.h"
#include "../SkillExecutionID.h"
#include "FluxioExecutor.h"

namespace armarx::skills
{
    class FluxioNativeExecutor : public FluxioExecutor
    {
    public:
        FluxioNativeExecutor(
            const std::string& id,
            const skills::SkillID& skillId,
            const FluxioSkill& fluxioSkill,
            const std::function<bool(const skills::SkillExecutionID&)>&& abortSkillFunc,
            const std::function<skills::SkillExecutionID(const skills::SkillExecutionRequest&)>&&
                executeSkillAsyncFunc,
            const std::function<std::optional<skills::SkillStatusUpdate>(
                const skills::SkillExecutionID&)>&& getSkillExecutionStatusFunc);

        void run(std::string executorName,
                 armarx::aron::data::DictPtr parameters,
                 std::experimental::observer_ptr<const FluxioProfile> profilePtr) override;
        void abort() override;
        std::optional<std::vector<skills::FluxioSkillStatusUpdate>> getStatusUpdate() override;
        armarx::aron::data::DictPtr getResultsCopy() override;

    private:
        std::optional<skills::SkillExecutionID> executionId = std::nullopt;
        const skills::SkillID skillId;
        const FluxioSkill& fluxioSkill;
        const armarx::Duration pollingFrequency = armarx::Duration::MilliSeconds(100);
        void setStatus(skills::SkillStatus status, const std::string& nodeId = "noId") override;
        const std::function<bool(const skills::SkillExecutionID&)> abortSkill;
        const std::function<skills::SkillExecutionID(const skills::SkillExecutionRequest&)>
            executeSkillAsync;
        const std::function<std::optional<skills::SkillStatusUpdate>(
            const skills::SkillExecutionID&)>
            getSkillExecutionStatus;
    };
} // namespace armarx::skills
